%%% FIG_REAL_TIME %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear memory
clear
clc
close all

% Set filename
fname = '';

% Add paths and directories
addpath(['..' filesep 'simulation' filesep 'v0' filesep 'functions'])
data_dir = ['..' filesep 'data' filesep 'output' filesep];
fig_dir  = ['output' filesep 'figures' filesep];
mat_dir  = ['output' filesep 'matfiles' filesep];
tab_dir  = ['output' filesep 'tables' filesep];

% Recover data and estimates
datasets  = load([data_dir 'data.mat']);
estimates = load([mat_dir sprintf('estimates_real_time%s.mat', fname)]);

% Extract objects for plot
real_time    = estimates.real_time;
time         = estimates.time;
state_save   = estimates.state_save;
state_median = squeeze(state_save(:, :, 50, :));

% Determine relevant signal
date_signal  = vintage2date( datasets.GDE.compr_aux );
n_month      = length(real_time);
signal_index = ones(n_month, 1);
for i_c = 2:length(date_signal)
    signal_index(real_time >= date_signal(i_c)) = i_c;
end

% Inputs for plots and tables
ax_fonts   = {'fontname', 'times', 'fontsize', 10, 'linewidth', 0.5};
black      = [  0,   0,   0];
blue       = [ 51,  51, 153]/255;
red        = [153,  51,  51]/255;
orange     = [242, 133,   0]/255;
green      = [ 51, 153,  51]/255;
dark_green = [  0, 100,   0]/255;
grey       = [ 51,  51,  51]/255;


%% PLOT STABILITY WITH RESPECT TO REVISIONS AND PANDEMIC

% 2014Q2 2014M7
IYEAR = 2014; IQUARTER = 2; JYEAR = 2014; JMONTH = 7; 
i_month     = find(year(real_time) == JYEAR & month(real_time) == JMONTH, 1, 'first'); 
i_quart     = find(year(time) == IYEAR & quarter(time) == IQUARTER, 1, 'first');
GDPsolera1n = state_median(signal_index(i_month), 1:i_quart, i_month);
time1n      = time(1:i_quart);
% 2015Q4 2016M1
IYEAR = 2015; IQUARTER = 4; JYEAR = 2016; JMONTH = 1; 
i_month     = find(year(real_time) == JYEAR & month(real_time) == JMONTH, 1, 'first'); 
i_quart     = find(year(time) == IYEAR & quarter(time) == IQUARTER, 1, 'first');
GDPsolera2n = state_median(signal_index(i_month), 1:i_quart, i_month);
time2n      = time(1:i_quart);
% 2017Q2 2017M7
IYEAR = 2017; IQUARTER = 2; JYEAR = 2017; JMONTH = 7; 
i_month     = find(year(real_time) == JYEAR & month(real_time) == JMONTH, 1, 'first'); 
i_quart     = find(year(time) == IYEAR & quarter(time) == IQUARTER, 1, 'first');
GDPsolera3n = state_median(signal_index(i_month), 1:i_quart,i_month);
time3n      = time(1:i_quart);
% 2018Q4 2019M1
IYEAR = 2018; IQUARTER = 4; JYEAR = 2019; JMONTH = 1; 
i_month     = find(year(real_time) == JYEAR & month(real_time) == JMONTH, 1, 'first'); 
i_quart     = find(year(time) == IYEAR & quarter(time) == IQUARTER, 1, 'first');
GDPsolera4n = state_median(signal_index(i_month), 1:i_quart, i_month);
time4n      = time(1:i_quart);
% 2020Q2 2020M7
IYEAR = 2020; IQUARTER = 2; JYEAR = 2020; JMONTH = 7; 
i_month     = find(year(real_time) == JYEAR & month(real_time) == JMONTH, 1, 'first'); 
i_quart     = find(year(time) == IYEAR & quarter(time) == IQUARTER, 1, 'first');
GDPsolera5n = state_median(signal_index(i_month),1:i_quart,i_month);
time5n      = time(1:i_quart);
% 2021Q4 2022M1
IYEAR = 2021; IQUARTER = 4; JYEAR = 2022; JMONTH = 1; 
i_month     = find(year(real_time) == JYEAR & month(real_time) == JMONTH, 1, 'first'); 
i_quart     = find(year(time) == IYEAR & quarter(time) == IQUARTER, 1, 'first');
GDPsolera6n = state_median(signal_index(i_month), 1:i_quart, i_month);
time6n      = time(1:i_quart);

% Figure for stability with respect to comprehensive revisions
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time1n, GDPsolera1n, time2n, GDPsolera2n, time3n, GDPsolera3n, ...
             time4n, GDPsolera4n, time5n, GDPsolera5n, time6n, GDPsolera6n);
set(plot0, 'linewidth', 1.5);
styles = {'--'; ':'; '-'; '-.'; '--'; ':'};
set(plot0, {'linestyle'}, styles)
markers = {'p'; 'o'; '+'; 'd'; 'none'; 'none'};
set(plot0, {'marker'}, markers, 'markersize', 5)
set(plot0, {'color'}, {blue; blue; red; red; dark_green; dark_green})
set(ax0, ax_fonts{:})
xticks([datetime(2005, 2, 1) datetime(2010, 2, 1) datetime(2015, 2, 1) datetime(2020, 2, 1)])
xticklabels({'2005Q1', '2010Q1', '2015Q1', '2020Q1'})
xlim([datetime(2004, 1, 1) datetime(2022, 10, 1)])
line(ax0, ax0.XLim, [0 0], 'linestyle', '--', 'color', black)
ylim([-32 23])
legend('2004Q1-2014Q2', '2004Q1-2015Q4', '2004Q1-2017Q2', ...
       '2004Q1-2018Q4', '2004Q1-2020Q2', '2004Q1-2021Q4', 'Location', 'South')
print(fig0, [fig_dir 'solera_vintages_all.eps'], '-depsc')

% Figure for stability with respect to CoVID-19 pandemic
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time5n, GDPsolera5n, time6n, GDPsolera6n);
set(plot0, 'linewidth', 1.5) 
styles = {'--'; ':'};
set(plot0, {'linestyle'}, styles)
set(plot0, {'color'}, {dark_green; dark_green})
set(ax0, ax_fonts{:})
xticks([datetime(2005, 2, 1) datetime(2010, 2, 1) datetime(2015, 2, 1) datetime(2020, 2, 1)])
xticklabels({'2005Q1', '2010Q1', '2015Q1', '2020Q1'})
xlim([datetime(2004, 1, 1) datetime(2022, 10, 1)])
line(ax0, ax0.XLim, [0 0], 'linestyle', '--', 'color', black)
ylim([-32 23])
legend('2004Q1-2020Q2', '2004Q1-2021Q4', 'Location', 'Best')
print(fig0, [fig_dir 'solera_vintages_covid.eps'], '-depsc')


%% PLOT COMPARISON WITH GDO AND GDPPLUS

% Extract GDE, GDI and GDO data
GDE_data      = readtable([data_dir 'GDE_raw.csv']);
GDI_data      = readtable([data_dir 'GDI_raw.csv']);
GDP_time      = cellfun(@(s) datetime(str2double(s(1:4)), 3*str2double(s(6))-2, 1), GDE_data{:, 1});
GDE_growth    = GDE_data{:, 2:end};
GDE_growth    = [NaN(1, size(GDE_growth, 2)); 100*((GDE_growth(2:end, :)./GDE_growth(1:(end-1), :)).^4-1)];
GDE_growth    = GDE_growth((GDP_time >= min(time)) & (GDP_time <= max(time)), :);
GDI_growth    = GDI_data{:, 2:end};
GDI_growth    = [NaN(1, size(GDI_growth, 2)); 100*((GDI_growth(2:end, :)./GDI_growth(1:(end-1), :)).^4-1)];
GDI_growth    = GDI_growth((GDP_time >= min(time)) & (GDP_time <= max(time)), :);
GDO_growth    = NaN(size(GDE_data{:, 2:end}, 1), size(GDE_data{:, 2:end}, 2), 2);
GDO_growth(:, :, 1) = GDE_data{:, 2:end};
GDO_growth(:, :, 2) = GDI_data{:, 2:end};
GDO_growth    = mean(GDO_growth, 3, 'omitnan');
GDO_growth    = [NaN(1, size(GDO_growth, 2)); 100*((GDO_growth(2:end, :)./GDO_growth(1:(end-1), :)).^4-1)];
GDO_growth    = GDO_growth((GDP_time >= min(time)) & (GDP_time <= max(time)), :);
GDP_time      = GDP_time((GDP_time >= min(time)) & (GDP_time <= max(time)), :);
GDP_real_time = datetime(2002, 9-1+(1:size(GDE_growth, 2))', 1);

% Match GDPPlus and GDO releases
GDO_growth    = GDO_growth(:, ismember(GDP_real_time, real_time));
GDP_real_time = GDP_real_time(ismember(GDP_real_time, real_time));

% Extract GDPplus data
GDPplus_all       = xlsread([data_dir 'GDPplus_raw.xlsx']);
GDPplus_time      = datetime(1960, 3*(1:size(GDPplus_all, 1))-2, 1)';
GDPplus_real_time = datetime(2013, 7+([1:65, 67:size(GDPplus_all, 2)+1]), 1)';
GDPplus_all       = GDPplus_all(:, GDPplus_real_time <= max(GDP_real_time));
GDPplus_real_time = GDPplus_real_time(GDPplus_real_time <= max(GDP_real_time));

% Compute recent and selected quarters
timex            = time( (time >= datetime(1985, 1, 1) & time <= datetime(2022, 7, 1)) );
GDPsolera_recent = state_median(signal_index(end), (time >= datetime(1985, 1, 1) & time <= datetime(2022, 7, 1)), end);
GDPplus_recent   = GDPplus_all((GDPplus_time >= datetime(1985, 1, 1) & GDPplus_time <= datetime(2022, 7, 1)), end);
GDO_recent       = GDO_growth((GDP_time >= datetime(1985, 1, 1) & GDP_time <= datetime(2022, 7, 1)), end);
i_q_2020Q1       = find(year(time) == 2020 & quarter(time) == 1, 1, 'first');
i_m_2020Q1       = find(real_time > time(i_q_2020Q1) + calmonths(2), 1, 'first');
timex1           = real_time(real_time > time(i_q_2020Q1) + calmonths(2));
GDPsolera_covid1 = squeeze( state_median(signal_index(i_m_2020Q1), i_q_2020Q1, i_m_2020Q1:end) );
i_q_2020Q1       = find(year(GDPplus_time) == 2020 & quarter(GDPplus_time) == 1, 1, 'first');
i_m_2020Q1       = find(GDPplus_real_time > GDPplus_time(i_q_2020Q1) + calmonths(2), 1, 'first');
GDPplus_covid1   = GDPplus_all(i_q_2020Q1, i_m_2020Q1:end)';
i_q_2020Q1       = find(year(GDP_time) == 2020 & quarter(GDP_time) == 1, 1, 'first');
i_m_2020Q1       = find(GDP_real_time > GDP_time(i_q_2020Q1) + calmonths(2), 1, 'first');
GDO_covid1       = GDO_growth(i_q_2020Q1, i_m_2020Q1:end)';
i_q_2020Q2       = find(year(time) == 2020 & quarter(time) == 2, 1, 'first');
i_m_2020Q2       = find(real_time > time(i_q_2020Q2) + calmonths(2), 1, 'first');
timex2           = real_time(real_time > time(i_q_2020Q2) + calmonths(2));
GDPsolera_covid2 = squeeze( state_median(signal_index(i_m_2020Q2), i_q_2020Q2, i_m_2020Q2:end) );
i_q_2020Q2       = find(year(GDPplus_time) == 2020 & quarter(GDPplus_time) == 2, 1, 'first');
i_m_2020Q2       = find(GDPplus_real_time > GDPplus_time(i_q_2020Q2) + calmonths(2), 1, 'first');
GDPplus_covid2   = GDPplus_all(i_q_2020Q2, i_m_2020Q2:end)';
i_q_2020Q2       = find(year(GDP_time) == 2020 & quarter(GDP_time) == 2, 1, 'first');
i_m_2020Q2       = find(GDP_real_time > GDP_time(i_q_2020Q2) + calmonths(2), 1, 'first');
GDO_covid2       = GDO_growth(i_q_2020Q2, i_m_2020Q2:end)';

% Determine quarter indicators
i_quart_solera = find(time >= datetime(2013, 10, 1) & time <= datetime(2022, 7, 1));
i_quart_plus   = find(GDPplus_time >= datetime(2013, 10, 1) & GDPplus_time <= datetime(2022, 7, 1));
i_quart_o      = find(GDP_time >= datetime(2013, 10, 1) & GDP_time <= datetime(2022, 7, 1));
n_quart        = length(i_quart_solera);

% Compute one-month and three-month nowcasts
time1          = time(i_quart_solera);
time3          = time(i_quart_solera);
GDPsolera_now1 = NaN(n_quart, 1);
GDPsolera_now3 = NaN(n_quart, 1);
for i_quart = 1:n_quart
    i_quart_tmp = i_quart_solera(i_quart);
    i_month = find(real_time > time(i_quart_tmp) + calmonths(2), 1, 'first');
    GDPsolera_now1(i_quart) = state_median(signal_index(i_month), i_quart_tmp, i_month);
    i_month = find(real_time > time(i_quart_tmp) + calmonths(4), 1, 'first');
    GDPsolera_now3(i_quart) = state_median(signal_index(i_month), i_quart_tmp, i_month);
end
GDPplus_now1 = zeros(n_quart, 1);
GDPplus_now3 = zeros(n_quart, 1);
for i_quart = 1:n_quart
    i_quart_tmp = i_quart_plus(i_quart);
    i_month = find(GDPplus_real_time > GDPplus_time(i_quart_tmp) + calmonths(2), 1, 'first');
    GDPplus_now1(i_quart) = GDPplus_all(i_quart_tmp, i_month);
    i_month = find(GDPplus_real_time > GDPplus_time(i_quart_tmp) + calmonths(4), 1, 'first');
    GDPplus_now3(i_quart) = GDPplus_all(i_quart_tmp, i_month);
end
GDO_now1 = zeros(n_quart, 1);
GDO_now3 = zeros(n_quart, 1);
for i_quart = 1:n_quart
    i_quart_tmp = i_quart_o(i_quart);
    i_month = find(GDP_real_time > GDP_time(i_quart_tmp) + calmonths(2), 1, 'first');
    GDO_now1(i_quart) = GDO_growth(i_quart_tmp, i_month);
    i_month = find(GDP_real_time > GDP_time(i_quart_tmp) + calmonths(4), 1, 'first');
    GDO_now3(i_quart) = GDO_growth(i_quart_tmp, i_month);
end

% Plot figure for most recent historical estimates
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, timex, GDPplus_recent(:), timex, GDO_recent(:), timex, GDPsolera_recent(:));
xlim([timex(1) timex(end)])
ylim([-34 30]);
set(plot0, 'linewidth', 1.5, {'linestyle'}, {'-'; '-.'; ':'})
markers = {'o'; 'x'; 's'};
set(plot0, {'marker'}, markers, 'markersize', 1.5)
set(ax0, ax_fonts{:})
set(plot0, {'color'}, {grey; orange; dark_green});
set(plot0, 'linewidth', 2)
line(ax0, [timex(1) timex(end)], [0 0], 'linestyle', '--', 'color', [0, 0, 0])
legend('GDPplus', 'GDO', 'GDPsolera', 'Location', 'Best');
print(fig0, [fig_dir 'gdp_plus_o_solera_1.eps'], '-depsc')

% Plot figure for 2020Q1 and 2020Q2 comparison
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, timex1, GDPplus_covid1(:), '-', timex1, GDO_covid1(:), '-.', timex1, GDPsolera_covid1(:), ':', ...
             timex2, GDPplus_covid2(:), '-', timex2, GDO_covid2(:), '-.', timex2, GDPsolera_covid2(:), ':');
set(plot0, 'linewidth', 1.5)
markers = {'o'; 'x'; 's'; '*'; 'p'; 'd'};
set(plot0, {'marker'}, markers,'markersize', 3)
set(ax0, ax_fonts{:})
set(plot0, {'color'}, {grey; orange; dark_green; grey; orange; dark_green})
legend('GDPplus Q1', 'GDO Q1', 'GDPsolera Q1', 'GDPplus Q2', 'GDO Q2', 'GDPsolera Q2', 'Location', 'West')    
xticks(datetime(2020, 3, 28) + calmonths([0, 5, 10, 15, 20, 25, 30]))
xticklabels({'Apr-2020', 'Sep-2020', 'Feb-2021', 'Jul-2021', 'Dec-2021', 'May-2022', 'Oct-2022'})
xlim([datetime(2020, 1, 1) datetime(2022, 12, 1)])
ylim([-40 0])
print(fig0, [fig_dir 'gdp_plus_o_solera_2.eps'], '-depsc')

% Plot figure for nowcast one-month after the quarter
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time1, GDPplus_now1, '-*', time1, GDPsolera_now1, ':o');
set(plot0, 'linewidth', 1.5, {'linestyle'}, {'-'; ':'})
set(ax0, ax_fonts{:})
set(plot0, {'color'}, {grey; dark_green})
xticks(datetime(2014:2:2022, 8, 1))
xticklabels({'2014Q3', '2016Q3', '2018Q3', '2020Q3', '2022Q3'})
xlim([datetime(2013, 7, 1) datetime(2022, 12, 1)])
line(ax0, ax0.XLim, [0 0], 'linestyle', '--', 'color', black)
ylim([-34 30])
legend('GDPplus', 'GDPsolera', 'Location', 'SouthWest')
print(fig0, [fig_dir 'gdp_nowcast_plus_1.eps'], '-depsc')

% Plot figure for nowcast one-month after the quarter
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time1, GDO_now1, '-*', time1, GDPsolera_now1, ':o');
set(plot0, 'linewidth', 1.5, {'linestyle'}, {'-.'; ':'})
set(ax0, ax_fonts{:})
set(plot0, {'color'}, {orange; dark_green})
xticks(datetime(2014:2:2022, 8, 1))
xticklabels({'2014Q3', '2016Q3', '2018Q3', '2020Q3', '2022Q3'})
xlim([datetime(2013, 7, 1) datetime(2022, 12, 1)])
line(ax0, ax0.XLim, [0 0], 'linestyle', '--', 'color', black)
ylim([-34 30])
legend('GDO', 'GDPsolera', 'Location', 'SouthWest')
print(fig0, [fig_dir 'gdp_nowcast_o_1.eps'], '-depsc')

% Plot figure for nowcast one-month after the quarter
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time1, GDO_now1, '-*', time1, GDPplus_now1, '-x', time1, GDPsolera_now1, ':o');
set(plot0, 'linewidth', 1.5, {'linestyle'}, {'-.'; '-'; ':'})
set(ax0, ax_fonts{:})
set(plot0, {'color'}, {orange; grey; dark_green})
xticks(datetime(2014:2:2022, 8, 1))
xticklabels({'2014Q3', '2016Q3', '2018Q3', '2020Q3', '2022Q3'})
xlim([datetime(2013, 7, 1) datetime(2022, 12, 1)])
line(ax0, ax0.XLim, [0 0], 'linestyle', '--', 'color', black)
ylim([-34 30])
legend('GDO', 'GDPplus', 'GDPsolera', 'Location', 'SouthWest')
print(fig0, [fig_dir 'gdp_nowcast_oplus_1.eps'], '-depsc')

% Plot figure for nowcast three-month after the quarter
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time3, GDPplus_now3, '-*', time3, GDPsolera_now3, ':o');
set(plot0, 'linewidth', 1.5, {'linestyle'}, {'-'; ':'})
set(ax0, ax_fonts{:})
set(plot0, {'color'}, {grey; dark_green})
xticks(datetime(2014:2:2022, 8, 1))
xticklabels({'2014Q3', '2016Q3', '2018Q3', '2020Q3', '2022Q3'})
xlim([datetime(2013, 7, 1) datetime(2022, 12, 1)])
line(ax0, ax0.XLim, [0 0], 'linestyle', '--', 'color', black)
ylim([-34 30])
legend('GDPplus', 'GDPsolera', 'Location', 'SouthWest')
print(fig0, [fig_dir 'gdp_nowcast_plus_3.eps'], '-depsc')

% Plot figure for nowcast three-month after the quarter
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time3, GDO_now3, '-*', time3, GDPsolera_now3, ':o');
set(plot0, 'linewidth', 1.5, {'linestyle'}, {'-.'; ':'})
set(ax0, ax_fonts{:})
set(plot0, {'color'}, {orange; dark_green})
xticks(datetime(2014:2:2022, 8, 1))
xticklabels({'2014Q3', '2016Q3', '2018Q3', '2020Q3', '2022Q3'})
xlim([datetime(2013, 7, 1) datetime(2022, 12, 1)])
line(ax0, ax0.XLim, [0 0], 'linestyle', '--', 'color', black)
ylim([-34 30])
legend('GDO', 'GDPsolera', 'Location', 'SouthWest')
print(fig0, [fig_dir 'gdp_nowcast_o_3.eps'], '-depsc')

% Plot figure for nowcast three-month after the quarter
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time3, GDO_now3, '-*', time3, GDPplus_now3, '-x', time3, GDPsolera_now3, ':o');
set(plot0, 'linewidth', 1.5, {'linestyle'}, {'-.'; '-'; ':'})
set(ax0, ax_fonts{:})
set(plot0, {'color'}, {orange; grey; dark_green})
xticks(datetime(2014:2:2022, 8, 1))
xticklabels({'2014Q3', '2016Q3', '2018Q3', '2020Q3', '2022Q3'})
xlim([datetime(2013, 7, 1) datetime(2022, 12, 1)])
line(ax0, ax0.XLim, [0 0], 'linestyle', '--', 'color', black)
ylim([-34 30])
legend('GDO', 'GDPplus', 'GDPsolera', 'Location', 'SouthWest')
print(fig0, [fig_dir 'gdp_nowcast_oplus_3.eps'], '-depsc')


%% PLOT SELECTED QUARTERS

% Set numerical parameters for real-time exercise
target_year_grid    = [2001, 2008, 2019];
target_quarter_grid = [   1,    4,    2]; 
n_periods           = length(target_year_grid);

for i_period = 1:n_periods

% Extract years    
target_year    = target_year_grid(i_period);
target_quarter = target_quarter_grid(i_period);

% Pair real-time indices and compute dimensions
estimates_F  = load([mat_dir sprintf('estimates_real_time_%dQ%d%s.mat', target_year, target_quarter, fname)]);
[~, pair_F]  = ismember(estimates_F.real_time, real_time);
state_save_F = estimates_F.state_save(:, :, :, ~(pair_F == 0));
real_time_F  = estimates_F.real_time(~(pair_F == 0));
pair_F(pair_F == 0) = [];
n_month      = length(pair_F);

% Assign growth rate estimates
i_quart_R  = find(year(estimates.time) == target_year & quarter(estimates.time) == target_quarter, 1, 'first');
i_quart_F  = find(year(estimates_F.time) == target_year & quarter(estimates_F.time) == target_quarter, 1, 'first');
growth_R   = NaN(n_month, 3);
growth_F   = NaN(n_month, 3);
growth_GDE = NaN(n_month, 1);
growth_GDI = NaN(n_month, 1);
for i_month = 1:n_month
    signal_index_aux     = signal_index(pair_F(i_month));
    growth_F(i_month, :) = squeeze(state_save_F(signal_index_aux, i_quart_F, [5, 50, 95], i_month));
    growth_R(i_month, :) = squeeze(state_save(signal_index_aux, i_quart_R, [5, 50, 95], pair_F(i_month)));
    data_index_aux       = find(real_time_F(i_month) == GDP_real_time, 1, 'first');
    if ~isempty(data_index_aux)
        growth_GDE(i_month)  = GDE_growth(i_quart_R, data_index_aux);
        growth_GDI(i_month)  = GDI_growth(i_quart_R, data_index_aux);  
    end
end

% Remove GDE and GDI data if they are not new releases
growth_GDE([false; (abs(diff(growth_GDE, 1, 1)) < 1e-2)]) = NaN;
growth_GDI([false; (abs(diff(growth_GDI, 1, 1)) < 1e-2)]) = NaN;

% Create figure for real time exercise
fig0  = figure();
ax0   = axes();
plot0 = fill(ax0, [real_time_F; flipud(real_time_F)], [growth_F(:, 1); flipud(growth_F(:, 3))], ...
    dark_green, 'facealpha', 0.1, 'linestyle', '--', 'edgecolor', green);
hold('on')
plot1 = fill(ax0, [real_time_F; flipud(real_time_F)], [growth_R(:, 1); flipud(growth_R(:, 3))], ...
    dark_green, 'FaceAlpha', 0.3, 'linestyle', '-', 'edgecolor', green);
hold('on')
plot2 = plot(ax0, real_time_F, [growth_R(:, 2), growth_F(:, 2), growth_GDE, growth_GDI]);
hold('off')

% Add extras
line(ax0, [real_time_F(1), real_time_F(end)], [0, 0], 'linewidth', 1, 'linestyle', ':', 'color', [0, 0, 0])
grid(ax0, 'on')
xlim(ax0, [real_time_F(1), real_time_F(end)])
ylim(ax0, [min([growth_F(:); growth_R(:); growth_GDE; growth_GDI])-0.5, ...
           max([growth_F(:); growth_R(:); growth_GDE; growth_GDI])+0.5])

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')

% Tune plot
set(plot2, 'linewidth', 1.5)
set(plot2, {'linestyle'}, {'-'; '--'; 'none'; 'none'})
set(plot2, {'color'}, {green; green; blue; red})
set(plot2, {'marker'}, {'none'; 'none'; 'x'; 'd'})
set(plot2, 'markersize', 7)
legend(plot2, {'GDPsolera (w/re-estimation)','GDPsolera (w/fixed parameters)', 'GDE', 'GDI'}, ...
    'orientation', 'vertical', 'location', 'best')

% Save and close figure
print(fig0, [fig_dir sprintf('real_time_%dQ%d.eps', target_year, target_quarter)], '-depsc')

end


%% TEST ON COVARIANCE RESTRICTIONS

% Load GDE data and compute growth rates
GDE_data   = readtable([data_dir 'GDE_raw.csv']);
GDP_time   = cellfun(@(s) datetime(str2double(s(1:4)), 3*str2double(s(6))-2, 1), GDE_data{:, 1});
GDE_growth = GDE_data{:, 2:end};
GDE_growth = [NaN(1, size(GDE_growth, 2)); 100*((GDE_growth(2:end, :)./GDE_growth(1:(end-1), :)).^4-1)];
GDE_growth = GDE_growth((GDP_time >= min(time)) & (GDP_time <= max(time)), :);

% Load GDI data and compute growth rates
GDI_data   = readtable([data_dir 'GDI_raw.csv']);
GDI_growth = GDI_data{:, 2:end};
GDI_growth = [NaN(1, size(GDI_growth, 2)); 100*((GDI_growth(2:end, :)./GDI_growth(1:(end-1), :)).^4-1)];
GDI_growth = GDI_growth((GDP_time >= min(time)) & (GDP_time <= max(time)), :);

% Compute release dates (first date is Sep-2002)
GDP_time      = GDP_time((GDP_time >= min(time)) & (GDP_time <= max(time)), :);
GDP_real_time = datetime(2002, 9-1+(1:size(GDE_growth, 2)), 1);

% Construct data for test of L2-optimality (begin 2002Q3/Oct-2002)
time_aux      = (GDP_time >= datetime(2002, 7, 1) & GDP_time < datetime(2022, 10, 1));
GDP_time      = GDP_time(time_aux);
real_time_aux = (GDP_real_time >= datetime(2002, 10, 1) & GDP_real_time < datetime(2023, 1, 1));
GDP_real_time = GDP_real_time(real_time_aux);
T_aux         = nnz(time_aux);
GDE_growth    = GDE_growth(time_aux, real_time_aux);
GDI_growth    = GDI_growth(time_aux, real_time_aux);
GDE_early     = NaN(T_aux, 3);
GDI_early     = NaN(T_aux, 2);
for j = 1:T_aux
    GDE_early(j, :) = GDE_growth(j, 3*(j-1)+(1:3));
    GDI_early(j, :) = GDI_growth(j, 3*(j-1)+(2:3));
end

% Delete pandemic observations
GDE_early(year(GDP_time) == 2020 & quarter(GDP_time) == 2, :) = NaN;
GDE_early(year(GDP_time) == 2020 & quarter(GDP_time) == 3, :) = NaN;
GDI_early(year(GDP_time) == 2020 & quarter(GDP_time) == 2, :) = NaN;
GDI_early(year(GDP_time) == 2020 & quarter(GDP_time) == 3, :) = NaN;

% Compute correlations of third estimates with revision errors
compute_reg = @(GDE, GDI) [var(GDE(:, 2)-GDE(:, 1), 'omitnan')\( [0, 1]*cov([GDE(:, 2), GDE(:, 2)-GDE(:, 1)], 'omitrows')*[1; 0] ); ...
                            cov([GDE(:, 3)-GDE(:, 2), GDE(:, 2)-GDE(:, 1)], 'omitrows')\( [0, 1, 0; 0, 0, 1]*cov([GDE(:, 3), GDE(:, 3)-GDE(:, 2), GDE(:, 2)-GDE(:, 1)], 'omitrows')*[1; 0; 0] ); ...
                            var(GDI(:, 2)-GDI(:, 1), 'omitnan')\( [0, 1]*cov([GDI(:, 2), GDI(:, 2)-GDI(:, 1)], 'omitrows')*[1; 0] )];
corr_reg    = compute_reg(GDE_early, GDI_early);
    
% Run bootstrap
rng(18)
n_boot    = 10000;
reg_boot = NaN(4, n_boot);
for i_boot = 1:n_boot
    units_boot = datasample(reshape((1:T_aux), 3, [])', T_aux/3); 
    reg_boot(:, i_boot) = compute_reg(GDE_early(units_boot(:), :), GDI_early(units_boot(:), :)); 
end

% Compute confidence intervals and test statistics
reg_interval = quantile(reg_boot, [0.025, 0.975], 2);
test_GDE_2   = corr_reg(1)/std(reg_boot(1, :), [], 2);
test_GDE_3   = (corr_reg(2:3)')*(cov(reg_boot(2:3, :)')\corr_reg(2:3));
test_GDI_3   = corr_reg(4)/std(reg_boot(4, :), [], 2);


%% WEIGHTS

N_E = estimates.dims(2);
N_I = estimates.dims(3);

% Extract years    
target_year    = 2014;
target_quarter = 3;

% Extract real-time indices and target quarter inex
estimates_F = load([mat_dir sprintf('estimates_real_time_%dQ%d%s.mat', target_year, target_quarter, fname)]);
time_F      = estimates_F.time;
i_target    = find(year(time_F) == target_year & quarter(time_F) == target_quarter, 1, 'first');
real_time_F = estimates_F.real_time;

% Extract weights for relevant index signal and set missing weights to zero
dims_weight         = size(estimates_F.weight_level_mean);
N_F = dims_weight(2); T_F = dims_weight(3); n_month = dims_weight(4);
weight_level_F      = NaN(dims_weight(2:4));
weight_diff_F       = NaN(dims_weight(2:4));
for i_month = 1:n_month
    i_signal                      = signal_index(find(real_time_F(i_month) == real_time, 1, 'first')); 
    weight_level_F(:, :, i_month) = estimates_F.weight_level_mean(i_signal, :, :, i_month);
    weight_diff_F(:, :, i_month)  = estimates_F.weight_diff_mean(i_signal, :, :, i_month);
end
weight_level_F(isnan(weight_level_F(:))) = 0;
weight_diff_F(isnan(weight_diff_F(:))) = 0;

% Select months to report
sel_months = [1:10, 22, 34]';
sel_months(sel_months > n_month) = [];

% Set names
weight_name      = cell(1, 9+2*21);
weight_name(1:9) = {'Total', 'Static', 'Dynamic', 'Past', 'Future', 'GDETotal', 'GDITotal', 'GDEStatic', 'GDIStatic'};
name_base        = {'GDEAdvance', 'GDESecond', 'GDEThird', 'GDEAOne',  'GDEATwo',  'GDEAThree', ...
                    'GDECOne', 'GDECTwo', 'GDECThree', 'GDECFour', 'GDECFive', ...
                    'GDISecond', 'GDIThird', 'GDIAOne',  'GDIATwo',  'GDIAThree', ...
                    'GDICOne', 'GDICTwo', 'GDICThree', 'GDICFour', 'GDICFive'};
for j = 1:length(name_base)
    weight_name{9+j}    = [name_base{j} 'Total'];
    weight_name{9+21+j} = [name_base{j} 'Static'];
end

% Compute sums of weights
weight_F       = weight_level_F;
sum_total      = reshape(sum(sum(weight_F, 2), 1), [n_month, 1]);
sum_static     = reshape(sum(weight_F(:, i_target, :), 1), [n_month, 1]);
sum_dynamic    = sum_total - sum_static; 
sum_past       = reshape(sum(sum(weight_F(:, 1:(i_target-1), :), 2), 1), [n_month, 1]);
sum_future     = sum_dynamic - sum_past;
sum_CS_total   = reshape(sum(weight_level_F, 2), [N_F, n_month])';
sum_CS_static  = reshape(weight_F(:, i_target, :), [N_F, n_month])';
sum_GDE_total  = sum(sum_CS_total(:, 1:N_E), 2);
sum_GDI_total  = sum(sum_CS_total(:, N_E+(1:N_I)), 2);
sum_GDE_static = sum(sum_CS_static(:, 1:N_E), 2);
sum_GDI_static = sum(sum_CS_static(:, N_E+(1:N_I)), 2);

% Display results in a table
tab_weight       = array2table(round([sum_total, sum_static, sum_dynamic, sum_past, sum_future, ...
    sum_GDE_total, sum_GDI_total, sum_GDE_static, sum_GDI_static, sum_CS_total, sum_CS_static], 2), 'variablenames', weight_name);
tab_weight.Date  = real_time_F;
tab_weight.Month = (1:n_month)';
tab_weight       = [tab_weight(:, 'Date'), tab_weight(:, 'Month'), tab_weight(:, 1:(end-2))];
writetable(tab_weight, [tab_dir sprintf('weight_level_all%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)
writetable(tab_weight(sel_months, :), [tab_dir sprintf('weight_level_select%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)

% Compute sums of weights
weight_F       = weight_diff_F;
sum_total      = reshape(sum(sum(weight_F, 2), 1), [n_month, 1]);
sum_static     = reshape(sum(weight_F(:, i_target, :), 1), [n_month, 1]);
sum_dynamic    = sum_total - sum_static; 
sum_past       = reshape(sum(sum(weight_F(:, 1:(i_target-1), :), 2), 1), [n_month, 1]);
sum_future     = sum_dynamic - sum_past;
sum_CS_total   = reshape(sum(weight_level_F, 2), [N_F, n_month])';
sum_CS_static  = reshape(weight_F(:, i_target, :), [N_F, n_month])';
sum_GDE_total  = sum(sum_CS_total(:, 1:N_E), 2);
sum_GDI_total  = sum(sum_CS_total(:, N_E+(1:N_I)), 2);
sum_GDE_static = sum(sum_CS_static(:, 1:N_E), 2);
sum_GDI_static = sum(sum_CS_static(:, N_E+(1:N_I)), 2);

% Display results in a table
tab_weight       = array2table(round([sum_total, sum_static, sum_dynamic, sum_past, sum_future, ...
    sum_GDE_total, sum_GDI_total, sum_GDE_static, sum_GDI_static, sum_CS_total, sum_CS_static], 2), 'variablenames', weight_name);
tab_weight.Date  = real_time_F;
tab_weight.Month = (1:n_month)';
tab_weight       = [tab_weight(:, 'Date'), tab_weight(:, 'Month'), tab_weight(:, 1:(end-2))];
writetable(tab_weight, [tab_dir sprintf('weight_diff_all%s.csv', fname)], 'writevariablenames', true)
writetable(tab_weight(sel_months, :), [tab_dir sprintf('weight_diff_select%s.csv', fname)], 'writevariablenames', true)

% Compute sums of weights
weight_F       = zeros(N_F, T_F, n_month);
for t = 1:T_F
    weight_F(:, t, :) = sum(weight_diff_F(:, t:T_F, :), 2)/4;
end
sum_total      = reshape(sum(sum(weight_F, 2), 1), [n_month, 1]);
sum_static     = reshape(sum(weight_F(:, i_target, :), 1), [n_month, 1]);
sum_dynamic    = sum_total - sum_static; 
sum_past       = reshape(sum(sum(weight_F(:, 1:(i_target-1), :), 2), 1), [n_month, 1]);
sum_future     = sum_dynamic - sum_past;
sum_CS_total   = reshape(sum(weight_level_F, 2), [N_F, n_month])';
sum_CS_static  = reshape(weight_F(:, i_target, :), [N_F, n_month])';
sum_GDE_total  = sum(sum_CS_total(:, 1:N_E), 2);
sum_GDI_total  = sum(sum_CS_total(:, N_E+(1:N_I)), 2);
sum_GDE_static = sum(sum_CS_static(:, 1:N_E), 2);
sum_GDI_static = sum(sum_CS_static(:, N_E+(1:N_I)), 2);

% Display results in a table
tab_weight       = array2table(round([sum_total, sum_static, sum_dynamic, sum_past, sum_future, ...
    sum_GDE_total, sum_GDI_total, sum_GDE_static, sum_GDI_static, sum_CS_total, sum_CS_static], 2), 'variablenames', weight_name);
tab_weight.Date  = real_time_F;
tab_weight.Month = (1:n_month)';
tab_weight       = [tab_weight(:, 'Date'), tab_weight(:, 'Month'), tab_weight(:, 1:(end-2))];
writetable(tab_weight, [tab_dir sprintf('weight_diff_diff_all%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)
writetable(tab_weight(sel_months, :), [tab_dir sprintf('weight_diff_diff_select%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)


%% FORECASTS

% Recover forecasts and data
forecast_mean = estimates.forecast_mean;
Y             = estimates.Y;
dims          = [5, 11, 10, 156];
N_E           = dims(2);
T             = dims(4);
name_series   = {'Advance', 'Second', 'Third', 'Annual 1', 'Annual 2', 'Annual 3'};

% Compute GDE data forecasts
id_real     = [2, 3, 4, 5, 6, 3, 4, 5, 6, 4, 5, 6];
id_naive    = [1, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3];
hor_fcast   = [1, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3];
n_fcast     = length(id_real);
name_fcast  = cell(n_fcast, 1);
for j = 1:n_fcast
    name_fcast{j} = [name_series{id_real(j)} ' (' num2str(hor_fcast(j)) '-month)'];
end
real_data   = Y(id_real, :)';
fcast_naive = Y(id_naive, :)';
fcast_model = NaN(T, n_fcast);
for t = 1:T
    for j = 1:n_fcast
        date_aux = datetime(year(time(t)), 3*quarter(time(t)), 1)+calmonths(hor_fcast(j));
        i_aux    = find(date_aux == real_time, 1, 'first');
        if ~isempty(i_aux)
            fcast_model(t, j) = forecast_mean(id_real(j), t, i_aux);
        end
    end
end
err_naive   = real_data - fcast_naive;
err_model   = real_data - fcast_model;

% Determine nonmissing entries and compute root mean-square errors
nonmiss    = ~isnan(fcast_model) & ~isnan(fcast_naive) & ~isnan(real_data);
n_RMSE     = NaN(n_fcast, 1);
RMSE_naive = NaN(n_fcast, 1);
RMSE_model = NaN(n_fcast, 1);
for j = 1:length(id_real)
    nonmiss(:, j) = nonmiss(:, j) & (min([err_naive(:, j), err_model(:, j)], [], 2) < 3.5); % prevent comparison for estimates of different signals
    n_RMSE(j)     = nnz(nonmiss(:, j));
    RMSE_naive(j) = sqrt(mean(err_naive(nonmiss(:, j), j).^2));
    RMSE_model(j) = sqrt(mean(err_model(nonmiss(:, j), j).^2));
end

% Create table with GDE forecast comparison
tab_fcast = table(name_fcast, n_RMSE, round(RMSE_naive, 2), round(RMSE_model, 2), 'VariableNames', {'Description', 'NObs', 'RMSEData', 'RMSEModel'});
disp(tab_fcast)
writetable(tab_fcast, [tab_dir sprintf('forecast_GDE%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)

% Display covariance matrices
disp('Covariance matrix of forecast errors - naive')
err_naive(~nonmiss(:)) = NaN;
disp(round(cov(err_naive, 'partialrows'), 4))
disp('Covariance matrix of forecast errors - model')
err_model(~nonmiss(:)) = NaN;
disp(round(cov(err_model, 'partialrows'), 4))


% Compute GDI forecasts
id_real     = N_E+[2, 3, 4, 5, 3, 4, 5];
id_naive    = N_E+[1, 1, 1, 1, 2, 2, 2];
hor_fcast   =   1+[1, 1, 1, 1, 2, 2, 2];
n_fcast     = length(id_real);
name_fcast  = cell(n_fcast, 1);
for j = 1:n_fcast
    name_fcast{j} = [name_series{id_real(j)-N_E+1} ' (' num2str(hor_fcast(j)) '-month)'];
end
real_data   = Y(id_real, :)';
fcast_naive = Y(id_naive, :)';
fcast_model = NaN(T, n_fcast);
for t = 1:T
    for j = 1:n_fcast
        date_aux = datetime(year(time(t)), 3*quarter(time(t)), 1)+calmonths(hor_fcast(j));
        i_aux    = find(date_aux == real_time, 1, 'first');
        if ~isempty(i_aux)
            fcast_model(t, j) = forecast_mean(id_real(j), t, i_aux);
        end
    end
end
err_naive   = real_data - fcast_naive;
err_model   = real_data - fcast_model;

% Determine nonmissing entries and compute root mean-square errors
nonmiss    = ~isnan(fcast_model) & ~isnan(fcast_naive) & ~isnan(real_data);
n_RMSE     = NaN(n_fcast, 1);
RMSE_naive = NaN(n_fcast, 1);
RMSE_model = NaN(n_fcast, 1);
for j = 1:length(id_real)
    nonmiss(:, j) = nonmiss(:, j) & (min([err_naive(:, j), err_model(:, j)], [], 2) < 3.5); % prevent comparison for estimates of different signals
    n_RMSE(j)     = nnz(nonmiss(:, j));
    RMSE_naive(j) = sqrt(mean(err_naive(nonmiss(:, j), j).^2));
    RMSE_model(j) = sqrt(mean(err_model(nonmiss(:, j), j).^2));
end

% Create table with GDE forecast comparison
tab_fcast = table(name_fcast, n_RMSE, round(RMSE_naive, 2), round(RMSE_model, 2), 'VariableNames', {'Description', 'NObs', 'RMSEData', 'RMSEModel'});
disp(tab_fcast)
writetable(tab_fcast, [tab_dir sprintf('forecast_GDI%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)

disp('Covariance matrix of forecast errors - naive')
err_naive(~nonmiss(:)) = NaN;
disp(round(cov(err_naive, 'partialrows'), 4))
disp('Covariance matrix of forecast errors - model')
err_model(~nonmiss(:)) = NaN;
disp(round(cov(err_model, 'partialrows'), 4))
