function [mu_X, rho_X, Sigma_X, ...
                rho_E, Sigma_E, ...
                rho_I, Sigma_I] = map_parameter(param, dims)
% MAP_PARAMETER  Extract parameter array of GDP solera model into 
%   individual parameters.
% 
%   [...] = MAP_PARAMETER(PARAM, DIMS) returns arrays assuming individual
%   parameters vertically concatenated into PARAM with dimensions 
%   DIMS = [N_C, N_E, N_I].
%
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c = dims(1);
N_E = dims(2);
N_I = dims(3);

% Create auxiliary index
n_param = [N_c, N_c, N_c^2, ...
                N_E, N_E^2, ...
                N_I, N_I^2];
idx_temp = [cumsum([1, n_param(1:(end-1))], 2); cumsum(n_param, 2)];

% Map parameters
mu_X    = param(idx_temp(1,  1):idx_temp(2,  1), :);
rho_X   = param(idx_temp(1,  2):idx_temp(2,  2), :);
Sigma_X = reshape(param(idx_temp(1,  3):idx_temp(2,  3), :), [N_c, N_c]);
rho_E   = param(idx_temp(1,  4):idx_temp(2,  4), :);
Sigma_E = reshape(param(idx_temp(1,  5):idx_temp(2,  5), :), [N_E, N_E]);
rho_I   = param(idx_temp(1,  6):idx_temp(2,  6), :);
Sigma_I = reshape(param(idx_temp(1, 7):idx_temp(2, 7), :), [N_I, N_I]);

end