function [mu_X, rho_X, lambda_X, sigma_X, ...
                rho_E, lambda_E, sigma_E, ...
                rho_I, lambda_I, sigma_I] = map_parameter(param, dims)
% MAP_PARAMETER  Extract parameter array of GDP solera model into 
%   individual parameters.
% 
%   [...] = MAP_PARAMETER(PARAM, DIMS) returns arrays assuming individual
%   parameters vertically concatenated into PARAM with dimensions 
%   DIMS = [N_C, N_E, N_I].
%
%   Version: 2020 Aug 06 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c = dims(1);
N_E = dims(2);
N_I = dims(3);

% Create auxiliary index
n_param = [N_c, N_c, N_c, N_c, ...
                N_E, N_E, N_E, ...
                N_I, N_I, N_I];
idx_temp = [cumsum([1, n_param(1:(end-1))], 2); cumsum(n_param, 2)];

% Map parameters
mu_X     = param(idx_temp(1,  1):idx_temp(2,  1), :);
rho_X    = param(idx_temp(1,  2):idx_temp(2,  2), :);
lambda_X = param(idx_temp(1,  3):idx_temp(2,  3), :);
sigma_X  = param(idx_temp(1,  4):idx_temp(2,  4), :);
rho_E    = param(idx_temp(1,  5):idx_temp(2,  5), :);
lambda_E = param(idx_temp(1,  6):idx_temp(2,  6), :);
sigma_E  = param(idx_temp(1,  7):idx_temp(2,  7), :);
rho_I    = param(idx_temp(1,  8):idx_temp(2,  8), :);
lambda_I = param(idx_temp(1,  9):idx_temp(2,  9), :);
sigma_I  = param(idx_temp(1, 10):idx_temp(2, 10), :);

end